@@ -11,14 +11,28 @@ from box import Box
 # Related modules
 from .. import common
 from ..data import get_from_box
+from ..data.validate import must_be_list
 from ..callback import Callback
-from ..augment.links import IFATTR
 from ..augment import devices
 
 # List of attributes we don't want propagated from defaults to global/node
 #
 no_propagate_list = ["attributes","extra_attributes","requires","supported_on","no_propagate","config_after","transform_after"]
 
+"""
+Return the authoritative list of all modules.
+
+A module is a top-level 'defaults' dictionary key if the value is a dictionary with 'supported_on' key
+"""
+def list_of_modules(topology: Box) -> list:
+  if not '_modlist' in topology.defaults._globals:
+    topology.defaults._globals._modlist = [
+      m for m in topology.defaults.keys() \
+        if isinstance(topology.defaults[m],dict) \
+          and 'supported_on' in topology.defaults[m] ]
+
+  return topology.defaults._globals._modlist
+
 class _Module(Callback):
 
   def __init__(self, data: Box) -> None:
@@ -33,6 +47,19 @@ class _Module(Callback):
     else:
       return _Module(data)
 
+"""
+pre_default: execute before any validation checks to set up any data structures that might affect other modules
+"""
+def pre_default(topology: Box) -> None:
+  adjust_modules(topology)
+  common.exit_on_error()
+
+  module_transform("pre_default",topology)
+  node_transform("pre_default",topology)
+  link_transform("pre_default",topology)
+  if 'module' in topology:
+    topology.defaults.module = topology.module
+
 """
 pre_transform: executed just before the main data model transformation is started
 
@@ -41,10 +68,7 @@ pre_transform: executed just before the main data model transformation is starte
 * Call module-specific link transformation code
 """
 def pre_transform(topology: Box) -> None:
-  adjust_modules(topology)
-  common.exit_on_error()
-
-  check_module_parameters(topology)
+  module_validate(topology)
   common.exit_on_error()
 
   module_transform("pre_transform",topology)
@@ -96,13 +120,22 @@ def post_transform(topology: Box) -> None:
 # Set default list of modules for nodes without specific module list
 #
 def augment_node_module(topology: Box) -> None:
-  if not 'module' in topology:
-    return
+  g_module = topology.get('module',[])
+  mod_list = sorted(list_of_modules(topology))
 
-  module = topology['module']
   for name,n in topology.nodes.items():
-    if not 'module' in n and n.get('role') != 'host' and devices.get_device_attribute(n,'role',topology.defaults) != 'host':
-      n.module = module
+    if 'module' in n:
+      must_be_list(
+        parent=n,
+        key='module',
+        path=f'nodes.{name}',
+        create_empty=False,
+        valid_values=mod_list)
+      continue
+
+    # Copy global modules (if they exist) into non-host devices
+    if g_module and n.get('role') != 'host' and devices.get_device_attribute(n,'role',topology.defaults) != 'host':
+      n.module = g_module
 
 # Check whether the modules defined on individual nodes are valid module names
 # and supported by the node device type
@@ -177,15 +210,26 @@ adjust_global_modules: last phase of global module adjustments
 * copy global settings into node settings (apart from no_propagate attributes)
 '''
 def adjust_global_modules(topology: Box) -> None:
-  mod_dict = { m : None for m in topology.get('module',[]) }
+  if 'module' in topology:
+    mod_list = sorted(list_of_modules(topology))
+    must_be_list(
+      parent=topology,
+      key='module',
+      path='topology',
+      create_empty=False,
+      valid_values=mod_list)
+
+  mods = list(topology.module)
   for name,n in topology.nodes.items():
-    for m in n.get('module',[]):
-      mod_dict[m] = None
+    if not 'module' in n:
+      continue
 
-  if not mod_dict:
-    return
+    mods.extend(n.module)
 
-  topology.module = list(mod_dict.keys())
+  if not mods:
+    topology.pop('module',None)
+  else:
+    topology.module = sorted(set(mods))
 
 """
 Merge default module parameters with global module parameters
@@ -254,119 +298,17 @@ def adjust_modules(topology: Box) -> None:
   if not 'module' in topology:
     return
     
+  common.exit_on_error()
   module_transform("init",topology)
   merge_node_module_params(topology)
   merge_global_module_params(topology)
   add_module_extra_parameters(topology)
-  check_module_parameters(topology)
+
+"""
+Validate module parameters and dependencies
+"""
+def module_validate(topology: Box) -> None:
   check_module_dependencies(topology)
-  module_transform("pre_default",topology)
-  node_transform("pre_default",topology)
-  link_transform("pre_default",topology)
-  if 'module' in topology:
-    topology.defaults.module = topology.module
-
-"""
-check_module_parameters:
-
-Verify global, node, link, and node-on-link data for all modules
-that include _attributes_ element in default settings
-
-parse_module_attributes:
-
-Accept list or dict as module attributes. List format applies to
-all levels (global, node, link, node-on-link), dict format specifies
-lists for every level.
-
-Global attributes are used as default value for node attributes.
-Link attributes are used as default value for node-on-link attributes.
-"""
-
-def check_module_parameters(topology: Box) -> None:
-  mod_attr = Box({},default_box=True,box_dots=True)
-
-  for m in topology.get("module",[]):                      # Iterate over all active modules
-    mod_def = topology.defaults.get(m,{})                  # Get module defaults
-    if mod_def:                                            # Did we get something meaningful?
-      if "attributes" in topology.defaults.get(m,{}):      # ... and does it include "attributes"?
-        mod_attr[m] = parse_module_attributes(topology.defaults[m].attributes)
-
-        if topology.get(m,{}):                             # Now we can start: are there global module parameters in topology?
-          for k in topology[m].keys():                     # Got them - iterate over them
-            if not k in mod_attr[m]["global"]:             # Did we get a parameter that is not in global attributes? Jeez... barf
-              common.error(
-                "Invalid global %s attribute %s" % (m,k),
-                common.IncorrectValue,
-                'module')
-
-  for name,n in topology.nodes.items():  # Inspect all nodes
-    for m in n.get("module",[]):         # Iterate over all node modules
-      if mod_attr[m] and m in n:         # Does the current module have a list of attributes?
-                                         # ...Does node have module attribute?
-        for k in n[m].keys():            # Iterate over node-level module-specific attributes
-          if not k in mod_attr[m].node:  # If the name of an attribute is not in the list of allowed
-                                         # ...node-level attributes report error
-            common.error(
-              f"Node {name}: invalid attribute {k} for module {m}",
-              common.IncorrectValue,
-              'module')
-
-  for g in topology.get('groups',{}):                    # Inspect node_data in groups
-    if 'node_data' in topology.groups[g]:
-      for m in topology.get('module',[]):                # Iterate over global modules
-        if m in topology.groups[g].node_data:            # Does the group node_data contain module attributes?
-          for k in topology.groups[g].node_data[m]:      # Iterate over module-specific attributes in node_data
-            if not k in mod_attr[m].node:                # Is the attribute a valid node attribute for the module?
-              common.error(
-                f"node_data in group {g} contains invalid attribute {k} for module {m}",
-                common.IncorrectValue,
-                'groups')
-
-  for l in topology.get("links",[]):        # Inspect all links - use get to avoid instantiating links key
-    for m in topology.get("module",[]):     # Iterate over all known modules - get avoids instantiating module key
-      if m in l and mod_attr[m] and l[m]:   # ... focusing on modules that have attributes specified on this link
-                                            # ... and want to have their attributes checked
-
-        # Module attribute could be a list of keys or a class type
-        #
-        if isinstance(l[m],Box) and isinstance(mod_attr[m].link,list):
-          for k in l[m].keys():             # Iterate over link-level module attributes
-            if not k in mod_attr[m].link:   # If the name of an attribute is not in the list of allowed
-                                            # ... link-level attributes, report an error
-              common.error(
-                f"Invalid attribute {k} for module {m} on link {l}",
-                common.IncorrectValue,
-                'modules')
-        else:
-          if type(l[m]).__name__ != str(mod_attr[m].link):
-            common.error(
-              f"Invalid value type for attribute {k} for module {m} on link {l}, expected {mod_attr[m].link}",
-              common.IncorrectValue,
-              'modules')
-
-
-    for intf in l.interfaces:                     # Iterate over all interfaces attached to the link
-      n = intf.node                               # ... get node name
-      node = topology.nodes[n]                    # ... and node data structure (so the expressions don't get too crazy)
-      for m in node.get("module",[]):             # Iterate over all node modules
-        if mod_attr[m] and m in intf:             # Does the current module have a list of attributes?
-                                                  # ... and interface contains module attributes?
-          # Module attribute could be a list of keys or a class type
-          #
-          if isinstance(intf[m],Box) and isinstance(mod_attr[m].link,list):
-            for k in intf[m].keys():              # Iterate over node link-level module-specific attributes
-              if not k in mod_attr[m].interface:  # If the name of an attribute is not in the list of allowed
-                                                  # ... interface attributes report error
-                common.error(
-                  f"Node {n} has invalid attribute {k} for module {m} on link {l}",
-                  common.IncorrectValue,
-                  'modules')
-          else:
-            if type(intf[m]).__name__ != str(mod_attr[m].interface) and not (intf[m] is False):
-              common.error(
-                f"Node {n} has invalid value type for module {m} on link {l}, expected {mod_attr[m].interface}",
-                common.IncorrectValue,
-                'modules')
 
 """
 Prepare module attribute dictionary
@@ -489,12 +431,13 @@ def copy_node_data_into_interfaces(topology: Box) -> None:
       for intf in n.get('interfaces',[]):                        # We might have some work to do, iterate over all interfaces
         if not isinstance(intf.get(m,{}),dict):                  # ... if the interface module data is not a dict, we can't merge
           continue
+        vrf_attr = Box({})                                       # Assume we have no VRF attributes
         if 'vrf' in intf and mod_attr.vrf_aware:                 # Do we have to deal with VRF-aware attributes?
-          vrf_attr = Box({ k: v
-            for k,v in n.vrfs[intf.vrf].get(m,{}).items()
-              if k in mod_attr.vrf_aware })                      # Build a Box of VRF attributes that could be copied to interfaces
-        else:
-          vrf_attr = Box({})                                     # ... otherwise set VRF attributes to an empty box
+          vrf_mod_data = n.vrfs[intf.vrf].get(m,{})
+          if not vrf_mod_data is False:                          # Deal with things like 'ospf: False' on VRF level
+            vrf_attr = Box({ k: v
+              for k,v in vrf_mod_data.items()
+                if k in mod_attr.vrf_aware })                    # Build a Box of VRF attributes that could be copied to interfaces
 
         if copy_attr or vrf_attr:                                # ... modify interface data only if we have something to merge
           intf[m] = copy_attr + vrf_attr + intf[m]
@@ -578,7 +521,7 @@ def link_transform(method: str, topology: Box) -> None:
 
   for l in topology.get("links",[]):
     mod_list: typing.Dict = {}
-    for node_data in l.get(IFATTR,[]):
+    for node_data in l.get('interfaces',[]):
       mod_list.update({ m: None for m in topology.nodes[node_data.node].get("module",[]) })
     for m in mod_list.keys():
       if not mod_load.get(m):  # pragma: no cover (module should have been loaded already)
