@@ -10,16 +10,20 @@ import netaddr
 from . import _Module,_routing
 from .. import common
 from .. import data
-from ..augment.links import IFATTR
+from ..data.validate import must_be_int,must_be_list,must_be_dict
 from ..augment import devices
 
 def check_bgp_parameters(node: Box) -> None:
   if not "bgp" in node:  # pragma: no cover (should have been tested and reported by the caller)
     return
   if not "as" in node.bgp:
-    common.error("Node %s has BGP enabled but no AS number specified" % node.name)
+    common.error(
+      f"Node {node.name} has BGP enabled but no AS number specified",
+      common.MissingValue,
+      'bgp')
+    return
 
-  data.must_be_int(parent=node,key='bgp.as',path=f'nodes.{node.name}',min_value=1,max_value=65535,module='bgp')
+  must_be_int(parent=node,key='bgp.as',path=f'nodes.{node.name}',min_value=1,max_value=65535,module='bgp')
 
   if "community" in node.bgp:
     bgp_comm = node.bgp.community
@@ -38,7 +42,7 @@ def check_bgp_parameters(node: Box) -> None:
       if not k in ['ibgp','ebgp']:
         common.error("Invalid BGP community setting in node %s: %s" % (node.name,k))
       else:
-        data.must_be_list(
+        must_be_list(
           parent=node.bgp.community,
           path=f'nodes.{node.name}.bgp.community',
           key=k,
@@ -58,7 +62,7 @@ def validate_bgp_sessions(node: Box, sessions: Box, attribute: str) -> bool:
         'bgp')
       OK = False
     else:
-      if data.must_be_list(
+      if must_be_list(
           parent=sessions,
           key=k,
           path=f'nodes.{node.name}.bgp.{attribute}',
@@ -255,9 +259,10 @@ def build_ebgp_sessions(node: Box, sessions: Box, topology: Box) -> None:
       if not ebgp_data is None:
         ebgp_data['as'] = neighbor_local_as
         if 'vrf' in l:        # VRF neighbor
-          if not node.vrfs[l.vrf].bgp.neighbors:
-            node.vrfs[l.vrf].bgp.neighbors = []
-          node.vrfs[l.vrf].bgp.neighbors.append(ebgp_data)
+          if not node.vrfs[l.vrf].bgp is False:
+            if not node.vrfs[l.vrf].bgp.neighbors:
+              node.vrfs[l.vrf].bgp.neighbors = []
+            node.vrfs[l.vrf].bgp.neighbors.append(ebgp_data)
         else:                 # Global neighbor
           node.bgp.neighbors.append(ebgp_data)
 
@@ -383,99 +388,69 @@ def bgp_set_advertise(node: Box, topology: Box) -> None:
     if l.get("type",None) in stub_roles or l.get("role",None) in stub_roles:
       l.bgp.advertise = True                # ... otherwise figure out whether to advertise the subnet
 
-class BGP(_Module):
+"""
+process_as_list:
+  If the global BGP parameters have as_list attribute, set node AS numbers and node
+  RR flags accordingly
+"""
+def process_as_list(topology: Box) -> None:
+  if not data.get_from_box(topology,'bgp.as_list'):         # Do we have global bgp.as_list setting?
+    return                                                  # ... nope, no work for me ;)) 
 
-  '''
-  process_as_list:
-    If the global BGP parameters have as_list attribute, set node AS numbers and node
-    RR flags accordingly
-  '''
-  def process_as_list(self, topology: Box) -> None:
-    if not 'bgp' in topology:                            # Do we have global BGP settings?
-      return
+  try:
+    must_be_dict(topology.bgp,'as_list','bgp',create_empty=False,module='bgp',abort=True)
+  except Exception as ex:
+    return
 
-    if not 'as_list' in topology.bgp:                    # Do we have global AS list?
-      return
-
-    if not isinstance(topology.bgp.as_list,dict):
+  node_data = Box({},default_box=True,box_dots=True)
+  node_list = list(topology.nodes.keys())
+  for asn,as_data in topology.bgp.as_list.items():
+    if not isinstance(as_data,Box):
       common.error(
-        "bgp.as_list should be a dictionary of AS parameters",
-        common.IncorrectValue)
-      return
+        f"Invalid value in bgp.as_list for ASN {asn}\n" + \
+        f"... Each ASN in a BGP as_list must be a dictionary with (at least) members key:\n"+
+        f"... Found: {as_data}",
+        common.IncorrectValue,
+        'bgp')
+      continue
 
-    node_data = Box({},default_box=True,box_dots=True)
-    for asn,as_data in topology.bgp.as_list.items():
-      if not isinstance(as_data,Box):
+    must_be_list(as_data,'members',f'bgp.as_list.{asn}',create_empty=False,module='bgp',valid_values=node_list)
+    must_be_list(as_data,'rr',f'bgp.as_list.{asn}',create_empty=False,module='bgp',valid_values=node_list)
+    if not as_data.members:
+      common.error(
+        f"BGP as_list for ASN {asn} does not have a valid list of members",
+        common.IncorrectValue,
+        'bgp')
+      continue
+
+    for n in as_data.members:
+      if 'as' in node_data[n]:
         common.error(
-          "Invalid value in bgp.as_list for ASN %s: " % asn + \
-          "\n... Each ASN in a BGP as_list must be a dictionary with (at least) members key:"+
-          "\n... Found: %s" % as_data,
+          f"BGP module supports at most 1 AS per node; {n} is already member of {node_data[n]['as']} and cannot also be part of {asn}",
+          common.IncorrectValue)
+        continue
+      node_data[n]["as"] = asn
+
+    for n in as_data.get('rr',{}):
+      if node_data[n]["as"] != asn:
+        common.error(
+          "Node %s is specified as route reflector in AS %s but is not in member list" % (n,asn),
+          common.IncorrectValue)
+        continue
+      node_data[n].rr = True
+
+  for name,node in topology.nodes.items():
+    if name in node_data:
+      node_as = node.bgp.get("as",None)
+      if node_as and node_as != node_data[name]["as"]:
+        common.error(
+          "Node %s has AS %s but is also in member list of AS %s" % (node.name,node_as,node_data[node.name]["as"]),
           common.IncorrectValue)
         continue
 
-      if not 'members' in as_data:
-        common.error(
-          "BGP as_list for ASN %s does not have a member attribute" % asn,
-          common.IncorrectValue)
-        continue
+      node.bgp = node_data[name] + node.bgp
 
-      data.must_be_list(as_data,'members',f'bgp.as_list.{asn}')
-      for n in as_data.members:
-        if not n in topology.nodes:
-          common.error(
-            "Invalid node name %s in member list of BGP AS %s" % (n,asn),
-            common.IncorrectValue)
-          continue
-        elif 'as' in node_data[n]:
-          common.error(
-            f"BGP module supports at most 1 AS per node; {n} is already member of {node_data[n]['as']} and cannot also be part of {asn}",
-            common.IncorrectValue)
-          continue
-        node_data[n]["as"] = asn
-
-      for n in as_data.get('rr',{}):
-        if not n in topology.nodes:
-          common.error(
-            "Invalid node name %s in route reflector list of BGP AS %s" % (n,asn),
-            common.IncorrectValue)
-          continue
-        if node_data[n]["as"] != asn:
-          common.error(
-            "Node %s is specified as route reflector in AS %s but is not in member list" % (n,asn),
-            common.IncorrectValue)
-          continue
-        node_data[n].rr = True
-
-    for name,node in topology.nodes.items():
-      if name in node_data:
-        node_as = node.bgp.get("as",None)
-        if node_as and node_as != node_data[name]["as"]:
-          common.error(
-            "Node %s has AS %s but is also in member list of AS %s" % (node.name,node_as,node_data[node.name]["as"]),
-            common.IncorrectValue)
-          continue
-
-        node.bgp = node_data[name] + node.bgp
-
-  '''
-  bgp_build_group: create automatic groups based on BGP AS numbers
-  '''
-  def build_bgp_groups(self, topology: Box) -> None:
-    for gname,gdata in topology.groups.items():
-      if re.match('as\\d+$',gname):
-        if gdata.get('members',None):
-          common.error('BGP AS groups should not have static members %s' % gname)
-
-    for name,node in topology.nodes.items():
-      if 'bgp' in node and 'as' in node.bgp:
-        grpname = "as%s" % node.bgp["as"]
-        if not grpname in topology.groups:
-          topology.groups[grpname] = { 'members': [] }
-
-        if not 'members' in topology.groups[grpname]:   # pragma: no cover (members list is created in group processing)
-          topology.groups[grpname].members = []
-
-        topology.groups[grpname].members.append(name)
+class BGP(_Module):
 
   """
   Module pre-default:
@@ -484,8 +459,7 @@ class BGP(_Module):
   * create automatic BGP groups
   """
   def module_pre_default(self, topology: Box) -> None:
-    self.process_as_list(topology)
-    self.build_bgp_groups(topology)
+    pass
 
   """
   Node pre-transform: set bgp.rr node attribute to _true_ if the node name is in the
@@ -497,6 +471,8 @@ class BGP(_Module):
       if node.name in topology.bgp.rr_list:
         node.bgp.rr = True
 
+    check_bgp_parameters(node)
+
   """
   Link pre-transform: Set link role based on BGP nodes attached to the link.
 
@@ -509,7 +485,7 @@ class BGP(_Module):
       return
 
     as_set = {}
-    for ifdata in link.get(IFATTR,[]):
+    for ifdata in link.get('interfaces',[]):
       n = ifdata.node
       if "bgp" in topology.nodes[n]:
         node_as = topology.nodes[n].bgp.get("as")
@@ -536,6 +512,6 @@ class BGP(_Module):
     if not "bgp" in node:   # pragma: no cover (this should have been caught in check_bgp_parameters)
       common.fatal(f"Internal error: node {node.name} has BGP module enabled but no BGP parameters","bgp")
       return
-    check_bgp_parameters(node)
     build_bgp_sessions(node,topology)
     bgp_set_advertise(node,topology)
+    _routing.remove_vrf_routing_blocks(node,'bgp')
