@@ -95,6 +95,10 @@ def validate_vlan_attributes(obj: Box, topology: Box) -> None:
     return
 
   for vname in list(obj.vlans.keys()):
+    if not isinstance(vname,str):
+      common.error(f'VLAN names in {obj_name} must be strings ({vname}):\n... {obj.vlans}',common.IncorrectValue,'vlan')
+      return
+
     if not obj.vlans[vname]:
       obj.vlans[vname] = Box({},default_box=True,box_dots=True)
 
@@ -328,10 +332,13 @@ def validate_trunk_vlan_list(link: Box) -> bool:
     if not 'vlan' in o_intf:                                          # Skip non-VLAN interfaces (obviously we have a native VLAN by now)
       continue
     if not 'trunk' in o_intf.vlan:                                    # a VLAN interface without a trunk attribute is a huge red flag
-      common.fatal('validate_trunk_vlan_list: Found a VLAN node without trunk attribute\n... {link}')
+      common.fatal(f'validate_trunk_vlan_list: Found a VLAN node without trunk attribute\n... {link}')
       return False
 
     for vname in o_intf.vlan.trunk.keys():                            # OK, now we can iterate over all VLANs in this interfaces' trunk
+      if not isinstance(vname,str):
+        common.error(f'VLAN names in trunks ({vname}) must be strings:\n... {link}',common.IncorrectValue,'vlan')
+        return False
       vlan_found = False
       for i_intf in link.interfaces:                                  # ... and over all other other interfaces
         if i_intf is o_intf or not 'vlan' in i_intf:                  # ... I did say all OTHER VLAN interfaces, right?
@@ -566,10 +573,16 @@ def create_loopback_vlan_links(topology: Box) -> None:
         topology.links.append(link_data)
 
 """
-get_vlan_data: Get VLAN data structure (node or topology)
+get_vlan_data: Get VLAN data structure (node or topology or interface neighbors)
 """
-def get_vlan_data(vlan: str, node: Box, topology: Box) -> typing.Optional[Box]:
-  return get_from_box(topology,f'vlans.{vlan}') or get_from_box(node,f'vlans.{vlan}')
+def get_vlan_data(vlan: str, node: Box, topology: Box, intf: Box) -> typing.Optional[Box]:
+  vlan_data = get_from_box(topology,f'vlans.{vlan}') or get_from_box(node,f'vlans.{vlan}')
+  if not vlan_data and 'neighbors' in intf:
+    for n in intf.neighbors:  # Look for local vlan in neighbor
+      vlan_data = get_from_box(topology.nodes[n.node],f'vlans.{vlan}')
+      if vlan_data:
+        break
+  return vlan_data
 
 """
 get_vlan_mode: Get VLAN mode attribute (node or topology), default 'irb'
@@ -581,7 +594,7 @@ def get_vlan_mode(node: Box, topology: Box) -> str:
 update_vlan_neighbor_list: Build a VLAN-wide list of neighbors
 """
 def update_vlan_neighbor_list(vlan: str, phy_if: Box, svi_if: Box, node: Box,topology: Box) -> None:
-  vlan_data = get_vlan_data(vlan,node,topology)                         # Try to get global or node-level VLAN data
+  vlan_data = get_vlan_data(vlan,node,topology,phy_if)                  # Try to get global or node-level VLAN data
   if vlan_data is None:                                                 # ... and get out if there's none
     return
 
@@ -740,13 +753,13 @@ set_svi_neighbor_list: set SVI neighbor list from VLAN neighbor list
 def set_svi_neighbor_list(node: Box, topology: Box) -> None:
   for ifdata in node.interfaces:
     if 'vlan_name' in ifdata:
-      vlan_data = get_vlan_data(ifdata.vlan_name,node,topology)             # Try to get global or local VLAN data
-      if vlan_data is None:                                                 # Not found?
-        continue                                                            # ... too bad
-
       if get_from_box(ifdata,'vlan.routed_link'):                           # Don't update neighbors on a routed VLAN link
         continue
 
+      vlan_data = get_vlan_data(ifdata.vlan_name,node,topology,ifdata)      # Try to get global or local VLAN data
+      if vlan_data is None:                                                 # Not found?
+        continue                                                            # ... too bad
+
       if not 'host_count' in vlan_data:                                     # Calculate the number of hosts attached to the VLAN
         vlan_data.host_count = len([ n for n in vlan_data.neighbors if topology.nodes[n.node].get('role','') == 'host' ])
 
