@@ -4,10 +4,11 @@
 import typing, re
 from box import Box
 
-from . import _Module,_routing,get_effective_module_attribute
+from . import _Module,_routing,_dataplane,get_effective_module_attribute
 from .. import common
 from .. import data
-from ..data import get_from_box,must_be_dict,global_vars
+from ..data import get_from_box,global_vars
+from ..data.validate import must_be_list,must_be_dict,validate_attributes
 from ..augment import devices,groups
 from .. import addressing
 
@@ -17,26 +18,16 @@ from .. import addressing
 #
 VALID_VRF_NAMES = re.compile( r"[a-zA-Z0-9_.-]{1,16}" )
 
-#
-# build_vrf_id_set: given an object (topology or node), create a set of RDs
-# that appear in that object.
-#
-def build_vrf_static_set(obj: Box, attr: str) -> set:
-  if 'vrfs' in obj:
-    return { v[attr] for v in obj.vrfs.values() if isinstance(v,dict) and attr in v and v[attr] is not None }
-  return set()
-
 def populate_vrf_static_ids(topology: Box) -> None:
-  vrf_data = global_vars.get('vrf')
+  for k in ('id','rd'):
+    _dataplane.create_id_set(f'vrf_{k}')
+    _dataplane.extend_id_set(f'vrf_{k}',_dataplane.build_id_set(topology,'vrfs',k,'topology'))
 
-  vrf_data.rd_set = build_vrf_static_set(topology,'rd')
-  vrf_data.id_set = build_vrf_static_set(topology,'id')
-  vrf_data.last_id = 1
+  _dataplane.set_id_counter('vrf_id',1,4095)
 
   for n in topology.nodes.values():
-    if 'vrfs' in n:
-      vrf_data.rd_set = vrf_data.rd_set.union(build_vrf_static_set(n,'rd'))
-      vrf_data.id_set = vrf_data.id_set.union(build_vrf_static_set(n,'id'))
+    for k in ('id','rd'):
+      _dataplane.extend_id_set(f'vrf_{k}',_dataplane.build_id_set(n,'vrfs',k,f'nodes.{n.name}'))
 
 #
 # Get a usable AS number. Try bgp.as then vrf.as from node and global settings
@@ -60,15 +51,17 @@ def parse_rdrt_value(value: str) -> typing.Optional[typing.List[int]]:
     return None
 
 def get_next_vrf_id(asn: str) -> typing.Tuple[int,str]:
-  vrf_data = global_vars.get('vrf')
+  rd_set = _dataplane.get_id_set('vrf_rd')
+  id_set = _dataplane.get_id_set('vrf_rd')
+  while True:
+    vrf_id = _dataplane.get_next_id('vrf_id')
+    if not f'{asn}:{vrf_id}' in rd_set:
+      break
 
-  while f'{asn}:{vrf_data.last_id}' in vrf_data.rd_set or vrf_data.last_id in vrf_data.id_set:
-    vrf_data.last_id = vrf_data.last_id + 1
-
-  rd = f'{asn}:{vrf_data.last_id}'
-  vrf_data.rd_set.add(rd)
-  vrf_data.id_set.add(vrf_data.last_id)
-  return (vrf_data.last_id,rd)
+  rd = f'{asn}:{vrf_id}'
+  rd_set.add(rd)
+  id_set.add(vrf_id)
+  return (vrf_id,rd)
 
 #
 # Check for 'reasonable' VRF names using a regex expression
@@ -207,7 +200,7 @@ def set_import_export_rt(obj : Box, topology: Box) -> None:
         vdata[rtname] = [ vdata.rd ]
         continue
 
-      data.must_be_list(vdata,rtname,f'{obj_id}.{vname}')
+      must_be_list(vdata,rtname,f'{obj_id}.{vname}')
 
       rtlist = []     # The final parsed and looked-up list of RT values
       for rtvalue in vdata[rtname]:
@@ -332,9 +325,33 @@ class VRF(_Module):
     if 'groups' in topology:
       groups.export_group_node_data(topology,'vrfs','vrf',copy_keys=['rd','export','import'])
 
-    if not 'vrfs' in topology:
+    if not must_be_dict(
+        parent=topology,
+        key='vrfs',
+        path='topology',
+        create_empty=False,
+        module='vrf'):                                # Check that we're dealing with a VRF dictionary and return if there's none
       return
 
+    for vname in topology.vrfs.keys():
+      must_be_dict(
+        parent=topology,
+        key=f'vrfs.{vname}',
+        path='topology',
+        create_empty=True,
+        module='vrf')
+
+      vdata = topology.vrfs[vname]
+      validate_attributes(
+        data=vdata,                                     # Validate global VRF data
+        topology=topology,
+        data_path=f'vrfs.{vname}',                      # Path to global VRF definition
+        data_name=f'VRF',
+        attr_list=['vrf','link'],                       # We're checking VLAN and link attributes
+        modules=topology.get('module',[]),              # ... against global modules
+        module_source='topology',
+        module='vrf')                                   # Function is called from 'vrf' module
+
     normalize_vrf_ids(topology)
     populate_vrf_static_ids(topology)
     set_vrf_ids(topology,topology)
@@ -348,10 +365,28 @@ class VRF(_Module):
         return
       node.vrfs = {}     # Prepare to pull in global vrfs
 
+    if not must_be_dict(
+        parent=node,
+        key='vrfs',
+        path=f'nodes.{node.name}',
+        create_empty=False,
+        module='vrf'):                                # Check that we're dealing with a VRF dictionary and return if there's none
+      return
+
     for vname in set(list(node.vrfs.keys()) + vlan_vrfs):  # Filter out duplicates
       if node.vrfs[vname] is None:
         node.vrfs[vname] = {}
 
+      validate_attributes(
+        data=node.vrfs[vname],                        # Validate node VRF data
+        topology=topology,
+        data_path=f'nodes.{node.name}.vrfs.{vname}',  # Path to node VRF definition
+        data_name=f'VRF',
+        attr_list=['vrf','link'],                     # We're checking VLAN and link attributes
+        modules=node.get('module',[]),                # ... against node modules
+        module_source=f'nodes.{node.name}',
+        module='vrf')                                 # Function is called from 'vrf' module
+
       if 'vrfs' in topology and vname in topology.vrfs:
         node.vrfs[vname] = topology.vrfs[vname] + node.vrfs[vname]
 
