@@ -6,6 +6,7 @@ from box import Box
 from . import _TopologyOutput
 from .. import providers
 from .. import common
+from ..augment import nodes
 
 class ProviderConfiguration(_TopologyOutput):
 
@@ -13,6 +14,10 @@ class ProviderConfiguration(_TopologyOutput):
     provider = providers._Provider.load(topology.provider,topology.defaults.providers[topology.provider])
     provider.call('pre_output_transform',topology)
 
+    # Creates a "ghost clean" topology after transformation
+    # (AKA, remove unmanaged devices)
+    provider_topology = nodes.ghost_buster(topology)
+
     filename = None
     if hasattr(self,'filenames'):
       filename = self.filenames[0]
@@ -22,4 +27,4 @@ class ProviderConfiguration(_TopologyOutput):
     if self.format:
       common.error('Specified output format(s) %s ignored' % self.format,common.IncorrectValue,'provider')
 
-    provider.create(topology,filename)
+    provider.create(provider_topology,filename)
